@@ -36,7 +36,7 @@ __docformat__ = 'restructuredtext'
 from roundup import date, hyperdb, password
 from roundup.backends import rdbms_common
 import MySQLdb
-import os, shutil
+import os, shutil, sys
 from MySQLdb.constants import ER
 import logging
 
@@ -49,11 +49,14 @@ isolation_levels = \
 
 def connection_dict(config, dbnamestr=None):
     d = rdbms_common.connection_dict(config, dbnamestr)
-    if d.has_key('password'):
+    if 'password' in d:
         d['passwd'] = d['password']
         del d['password']
-    if d.has_key('port'):
+    if 'port' in d:
         d['port'] = int(d['port'])
+    charset = config.RDBMS_MYSQL_CHARSET
+    if charset != 'default':
+        d['charset'] = charset
     return d
 
 def db_nuke(config):
@@ -63,7 +66,7 @@ def db_nuke(config):
         conn = MySQLdb.connect(**kwargs)
         try:
             conn.select_db(config.RDBMS_NAME)
-        except:
+        except MySQLdb.Error:
             # no, it doesn't exist
             pass
         else:
@@ -89,7 +92,9 @@ def db_create(config):
     kwargs = connection_dict(config)
     conn = MySQLdb.connect(**kwargs)
     cursor = conn.cursor()
-    command = "CREATE DATABASE %s"%config.RDBMS_NAME
+    command = "CREATE DATABASE %s COLLATE utf8_general_ci"%config.RDBMS_NAME
+    if sys.version_info[0] > 2:
+        command += ' CHARACTER SET utf8'
     logging.info(command)
     cursor.execute(command)
     conn.commit()
@@ -180,11 +185,11 @@ class Database(rdbms_common.Database):
         try:
             self.load_dbschema()
         except MySQLdb.OperationalError as message:
-            if message[0] != ER.NO_DB_ERROR:
+            if message.args[0] != ER.NO_DB_ERROR:
                 raise
         except MySQLdb.ProgrammingError as message:
-            if message[0] != ER.NO_SUCH_TABLE:
-                raise hyperdb.DatabaseError, message
+            if message.args[0] != ER.NO_SUCH_TABLE:
+                raise hyperdb.DatabaseError(message)
             self.init_dbschema()
             self.sql("CREATE TABLE `schema` (`schema` TEXT) ENGINE=%s"%
                 self.mysql_backend)
@@ -199,7 +204,8 @@ class Database(rdbms_common.Database):
         self.cursor.execute('select `schema` from `schema`')
         schema = self.cursor.fetchone()
         if schema:
-            self.database_schema = eval(schema[0])
+            # bandit - schema is trusted
+            self.database_schema = eval(schema[0])  # nosec
         else:
             self.database_schema = {}
 
@@ -228,8 +234,10 @@ class Database(rdbms_common.Database):
         self.sql('''CREATE TABLE __textids (_class VARCHAR(255),
             _itemid VARCHAR(255), _prop VARCHAR(255), _textid INT)
             ENGINE=%s'''%self.mysql_backend)
-        self.sql('''CREATE TABLE __words (_word VARCHAR(30),
-            _textid INT) ENGINE=%s'''%self.mysql_backend)
+        self.sql('''CREATE TABLE __words (_word VARCHAR(%s),
+            _textid INT) ENGINE=%s''' % ((self.indexer.maxlength + 5),
+                                         self.mysql_backend)
+            )
         self.sql('CREATE INDEX words_word_ids ON __words(_word)')
         self.sql('CREATE INDEX words_by_id ON __words (_textid)')
         self.sql('CREATE UNIQUE INDEX __textids_by_props ON '
@@ -252,12 +260,12 @@ class Database(rdbms_common.Database):
             for name, s_prop in old_spec[1]:
                 # s_prop is a repr() string of a hyperdb type object
                 if s_prop.find('Multilink') == -1:
-                    if properties.has_key(name):
+                    if name in properties:
                         propnames.append(name)
                     continue
                 tn = '%s_%s'%(cn, name)
 
-                if properties.has_key(name):
+                if name in properties:
                     # grabe the current values
                     sql = 'select linkid, nodeid from %s'%tn
                     self.sql(sql)
@@ -268,7 +276,7 @@ class Database(rdbms_common.Database):
                 sql = 'drop table %s'%tn
                 self.sql(sql)
 
-                if properties.has_key(name):
+                if name in properties:
                     # re-create and populate the new table
                     self.create_multilink_table(klass, name)
                     sql = '''insert into %s (linkid, nodeid) values
@@ -389,6 +397,30 @@ class Database(rdbms_common.Database):
         # Convert all String properties to TEXT
         self._convert_string_properties()
 
+    def fix_version_5_tables(self):
+        # A bug caused the _<class>_key_retired_idx to be missing
+        # unless the database was upgraded from version 4 to 5.
+        # If it was created at version 5, the index is missing.
+        # The user class is always present and has a key.
+        # Check it for the index. If missing, add index to all
+        # classes by rerunning self.fix_version_4_tables().
+
+        # if this fails abort. Probably means no user class
+        # so we should't be doing anything.
+        if not self.sql_index_exists("_user", "_user_key_retired_idx"):
+            self.fix_version_4_tables()
+        else:
+            self.log_info('No changes needed.')
+
+    def fix_version_6_tables(self):
+        # Modify length for __words._word column.
+        c = self.cursor
+        sql = "alter table __words change column _word _word varchar(%s)" % (
+                                                                   self.arg)
+        # Why magic number 5? It was the original offset between
+        #   column length and maxlength.
+        c.execute(sql, (self.indexer.maxlength + 5,))
+
     def __repr__(self):
         return '<myroundsql 0x%x>'%id(self)
 
@@ -440,6 +472,10 @@ class Database(rdbms_common.Database):
                         spec.classname, idx)
             self.sql(index_sql3)
 
+            # and the unique index for key / retired(id)
+            self.add_class_key_required_unique_constraint(spec.classname,
+                                                          spec.key)
+
         # TODO: create indexes on (selected?) Link property columns, as
         # they're more likely to be used for lookup
 
@@ -524,6 +560,12 @@ class Database(rdbms_common.Database):
         sql = 'drop index %s on %s'%(index_name, table_name)
         self.sql(sql)
 
+        # and now the retired unique index too
+        index_name = '_%s_key_retired_idx' % cn
+        if self.sql_index_exists(table_name, index_name):
+            sql = 'drop index %s on _%s'%(index_name, cn)
+            self.sql(sql)
+
     # old-skool id generation
     def newid(self, classname):
         ''' Generate a new id for the given class
@@ -583,22 +625,41 @@ class Database(rdbms_common.Database):
         self.log_info('close')
         try:
             self.conn.close()
+        # issue2551025: with revision 1.3.14 of mysqlclient.
+        # It looks like you can get an OperationalError 2006
+        # raised for closing a closed handle.
+        except MySQLdb.OperationalError as message:
+            if str(message) != "(2006, '')":  # close connection
+                raise
         except MySQLdb.ProgrammingError as message:
             if str(message) != 'closing a closed connection':
                 raise
 
 class MysqlClass:
+    case_sensitive_equal = 'COLLATE utf8_bin ='
 
-    def supports_subselects(self):
-        # TODO: AFAIK its version dependent for MySQL
-        return False
+    # TODO: AFAIK its version dependent for MySQL
+    supports_subselects = False
 
-    def _subselect(self, classname, multilink_table):
+    def _subselect(self, proptree):
         ''' "I can't believe it's not a toy RDBMS"
            see, even toy RDBMSes like gadfly and sqlite can do sub-selects...
         '''
-        self.db.sql('select nodeid from %s'%multilink_table)
-        s = ','.join([x[0] for x in self.db.sql_fetchall()])
+        classname       = proptree.parent.classname
+        multilink_table = proptree.propclass.table_name
+        nodeid_name     = proptree.propclass.nodeid_name
+        linkid_name     = proptree.propclass.linkid_name
+
+        w = ''
+        if proptree.need_retired:
+            w = ' where %s.__retired__=0'%(multilink_table)
+        if proptree.need_child_retired:
+            tn1 = multilink_table
+            tn2 = '_' + proptree.classname
+            w = ', %s where %s.%s=%s.id and %s.__retired__=0'%(tn2, tn1,
+                linkid_name, tn2, tn2)
+        self.db.sql('select %s from %s%s'%(nodeid_name, multilink_table, w))
+        s = ','.join([str(x[0]) for x in self.db.sql_fetchall()])
         return '_%s.id not in (%s)'%(classname, s)
 
     def create_inner(self, **propvalues):
@@ -660,7 +721,7 @@ class MysqlClass:
         # then the snapshot has already been established.
         if e[0] == ER.DUP_ENTRY:
             key = propvalues[self.key]
-            raise ValueError, 'node with key "%s" exists' % key
+            raise ValueError('node with key "%s" exists' % key)
         # We don't know what this exception is; reraise it.
         raise
         
