@@ -1,9 +1,10 @@
 import re, mimetypes
 
 from roundup import hyperdb, date, password
-from roundup.cgi import templating
+from roundup.cgi import templating, TranslationService
 from roundup.cgi.exceptions import FormError
 
+
 class FormParser:
     # edit form variable handling (see unit tests)
     FV_LABELS = r'''
@@ -37,11 +38,11 @@ class FormParser:
             self._ = self.gettext = client.gettext
             self.ngettext = client.ngettext
         except AttributeError:
-            _translator = templating.translationService
+            _translator = TranslationService.get_translation()
             self._ = self.gettext = _translator.gettext
             self.ngettext = _translator.ngettext
 
-    def parse(self, create=0, num_re=re.compile('^\d+$')):
+    def parse(self, create=0, num_re=re.compile(r'^\d+$')):
         """ Item properties and their values are edited with html FORM
             variables and their values. You can:
 
@@ -135,7 +136,7 @@ class FormParser:
                 they are valid for the class).  Otherwise, the property
                 is set to the form value.
 
-                For Date(), Interval(), Boolean(), and Number()
+                For Date(), Interval(), Boolean(), and Number(), Integer()
                 properties, the form value is converted to the
                 appropriate
 
@@ -163,13 +164,22 @@ class FormParser:
 
                 The String content value is handled as described above for
                 file uploads.
+                If "multiple" is turned on for file uploads in the html
+                template, multiple links are generated::
+
+                    @link@files=file-2
+                    file-2@content=value
+                    ...
+
+                depending on how many files the user has attached.
 
             If both the "@note" and "@file" form variables are
             specified, the action::
 
                     @link@msg-1@files=file-1
 
-            is also performed.
+            is also performed. If "multiple" is specified this is
+            carried out for each of the attached files.
 
             We also check that FileClass items have a "content" property with
             actual content, otherwise we remove them from all_props before
@@ -192,8 +202,8 @@ class FormParser:
             classes = '|'.join(db.classes.keys())
             # specials for parsePropsFromForm
             # handle the various forms (see unit tests)
-            self.FV_SPECIAL = re.compile(self.FV_LABELS%classes, re.VERBOSE)
-            self.FV_DESIGNATOR = re.compile(r'(%s)([-\d]+)'%classes)
+            self.FV_SPECIAL = re.compile(self.FV_LABELS % classes, re.VERBOSE)
+            self.FV_DESIGNATOR = re.compile(r'(%s)([-\d]+)' % classes)
 
         # these indicate the default class / item
         default_cn = self.classname
@@ -238,17 +248,14 @@ class FormParser:
                 nodeid = '-1'
                 propname = 'content'
                 all_links.append((default_cn, default_nodeid, 'messages',
-                    [('msg', '-1')]))
+                                  [('msg', '-1')]))
                 have_note = 1
             elif d['file']:
                 # the special file field
-                cn = 'file'
-                cl = self.db.classes[cn]
-                nodeid = '-1'
-                propname = 'content'
-                all_links.append((default_cn, default_nodeid, 'files',
-                    [('file', '-1')]))
-                have_file = 1
+                cn = default_cn
+                cl = default_cl
+                nodeid = default_nodeid
+                propname = 'files'
             else:
                 # default
                 cn = default_cn
@@ -265,13 +272,13 @@ class FormParser:
 
             # get more info about the class, and the current set of
             # form props for it
-            if not all_propdef.has_key(cn):
+            if cn not in all_propdef:
                 all_propdef[cn] = cl.getprops()
             propdef = all_propdef[cn]
-            if not all_props.has_key(this):
+            if this not in all_props:
                 all_props[this] = {}
             props = all_props[this]
-            if not got_props.has_key(this):
+            if this not in got_props:
                 got_props[this] = {}
 
             # is this a link command?
@@ -280,27 +287,27 @@ class FormParser:
                 for entry in self.extractFormList(form[key]):
                     m = self.FV_DESIGNATOR.match(entry)
                     if not m:
-                        raise FormError, self._('link "%(key)s" '
-                            'value "%(entry)s" not a designator') % locals()
+                        raise FormError(self._('link "%(key)s" '
+                            'value "%(entry)s" not a designator') % locals())
                     value.append((m.group(1), m.group(2)))
 
                     # get details of linked class
                     lcn = m.group(1)
                     lcl = self.db.classes[lcn]
                     lnodeid = m.group(2)
-                    if not all_propdef.has_key(lcn):
+                    if lcn not in all_propdef:
                         all_propdef[lcn] = lcl.getprops()
-                    if not all_props.has_key((lcn, lnodeid)):
+                    if (lcn, lnodeid) not in all_props:
                         all_props[(lcn, lnodeid)] = {}
-                    if not got_props.has_key((lcn, lnodeid)):
+                    if (lcn, lnodeid) not in got_props:
                         got_props[(lcn, lnodeid)] = {}
 
                 # make sure the link property is valid
                 if (not isinstance(propdef[propname], hyperdb.Multilink) and
                         not isinstance(propdef[propname], hyperdb.Link)):
-                    raise FormError, self._('%(class)s %(property)s '
-                        'is not a link or multilink property') % {
-                        'class':cn, 'property':propname}
+                    raise FormError(self._('%(class)s %(property)s '
+                            'is not a link or multilink property') % {
+                            'class':cn, 'property':propname})
 
                 all_links.append((cn, nodeid, propname, value))
                 continue
@@ -310,14 +317,14 @@ class FormParser:
                 for entry in self.extractFormList(form[key]):
                     m = self.FV_SPECIAL.match(entry)
                     if not m:
-                        raise FormError, self._('The form action claims to '
+                        raise FormError(self._('The form action claims to '
                             'require property "%(property)s" '
                             'which doesn\'t exist') % {
-                            'property':propname}
+                            'property':propname})
                     if m.group('classname'):
                         this = (m.group('classname'), m.group('id'))
                         entry = m.group('propname')
-                    if not all_required.has_key(this):
+                    if this not in all_required:
                         all_required[this] = []
                     all_required[this].append(entry)
                 continue
@@ -330,12 +337,12 @@ class FormParser:
                 mlaction = 'add'
 
             # does the property exist?
-            if not propdef.has_key(propname):
+            if propname not in propdef:
                 if mlaction != 'set':
-                    raise FormError, self._('You have submitted a %(action)s '
+                    raise FormError(self._('You have submitted a %(action)s '
                         'action for the property "%(property)s" '
                         'which doesn\'t exist') % {
-                        'action': mlaction, 'property':propname}
+                        'action': mlaction, 'property': propname})
                 # the form element is probably just something we don't care
                 # about - ignore it
                 continue
@@ -346,16 +353,21 @@ class FormParser:
             value = form[key]
 
             # handle unpacking of the MiniFieldStorage / list form value
-            if isinstance(proptype, hyperdb.Multilink):
+            if d['file']:
+                assert isinstance(proptype, hyperdb.Multilink)
+                # value is a file upload... we *always* handle multiple
+                # files here (html5)
+                if not isinstance(value, type([])):
+                    value = [value]
+            elif isinstance(proptype, hyperdb.Multilink):
                 value = self.extractFormList(value)
             else:
                 # multiple values are not OK
                 if isinstance(value, type([])):
-                    raise FormError, self._('You have submitted more than one '
-                        'value for the %s property') % propname
-                # value might be a file upload...
-                if not hasattr(value, 'filename') or value.filename is None:
-                    # nope, pull out the value and strip it
+                    raise FormError(self._('You have submitted more than one '
+                        'value for the %s property') % propname)
+                # value might be a single file upload
+                if not getattr(value, 'filename', None):
                     value = value.value.strip()
 
             # now that we have the props field, we need a teensy little
@@ -377,33 +389,60 @@ class FormParser:
                         confirm = form[key]
                         break
                 else:
-                    raise FormError, self._('Password and confirmation text '
-                        'do not match')
+                    raise FormError(self._('Password and confirmation text '
+                                           'do not match'))
                 if isinstance(confirm, type([])):
-                    raise FormError, self._('You have submitted more than one '
-                        'value for the %s property') % propname
+                    raise FormError(self._('You have submitted more than one '
+                        'value for the %s property') % propname)
                 if value != confirm.value:
-                    raise FormError, self._('Password and confirmation text '
-                        'do not match')
+                    raise FormError(self._('Password and confirmation text '
+                                           'do not match'))
                 try:
-                    value = password.Password(value, config=self.db.config)
-                except hyperdb.HyperdbValueError, msg:
-                    raise FormError, msg
+                    value = password.Password(value, scheme=proptype.scheme,
+                                              config=self.db.config)
+                except hyperdb.HyperdbValueError as msg:
+                    raise FormError(msg)
+            elif d['file']:
+                # This needs to be a Multilink and is checked above
+                fcn = 'file'
+                fcl = self.db.classes[fcn]
+                fpropname = 'content'
+                if fcn not in all_propdef:
+                    all_propdef[fcn] = fcl.getprops()
+                fpropdef = all_propdef[fcn]
+                have_file = []
+                for n, v in enumerate(value):
+                    if not hasattr(v, 'filename'):
+                        raise FormError(self._('Not a file attachment'))
+                    # skip if the upload is empty
+                    if not v.filename:
+                        continue
+                    fnodeid = str(-(n+1))
+                    have_file.append(fnodeid)
+                    fthis = (fcn, fnodeid)
+                    if fthis not in all_props:
+                        all_props[fthis] = {}
+                    fprops = all_props[fthis]
+                    all_links.append((cn, nodeid, 'files',
+                                      [('file', fnodeid)]))
 
+                    fprops['content'] = self.parse_file(fpropdef, fprops, v)
+                value = None
+                nodeid = None
             elif isinstance(proptype, hyperdb.Multilink):
                 # convert input to list of ids
                 try:
                     l = hyperdb.rawToHyperdb(self.db, cl, nodeid,
-                        propname, value)
-                except hyperdb.HyperdbValueError, msg:
-                    raise FormError, msg
+                                             propname, value)
+                except hyperdb.HyperdbValueError as msg:
+                    raise FormError(msg)
 
                 # now use that list of ids to modify the multilink
                 if mlaction == 'set':
                     value = l
                 else:
                     # we're modifying the list - get the current list of ids
-                    if props.has_key(propname):
+                    if propname in props:
                         existing = props[propname]
                     elif nodeid and not nodeid.startswith('-'):
                         existing = cl.get(nodeid, propname, [])
@@ -418,10 +457,10 @@ class FormParser:
                             try:
                                 existing.remove(entry)
                             except ValueError:
-                                raise FormError, self._('property '
+                                raise FormError(self._('property '
                                     '"%(propname)s": "%(value)s" '
                                     'not currently in list') % {
-                                    'propname': propname, 'value': entry}
+                                    'propname': propname, 'value': entry})
                     else:
                         # add - easy, just don't dupe
                         for entry in l:
@@ -430,44 +469,24 @@ class FormParser:
                     value = existing
                     # Sort the value in the same order used by
                     # Multilink.from_raw.
-                    value.sort(lambda x, y: cmp(int(x),int(y)))
+                    value.sort(key=int)
 
-            elif value == '':
+            elif value == '' or value == b'':
                 # other types should be None'd if there's no value
                 value = None
             else:
                 # handle all other types
                 try:
-                    if isinstance(proptype, hyperdb.String):
-                        if (hasattr(value, 'filename') and
-                                value.filename is not None):
-                            # skip if the upload is empty
-                            if not value.filename:
-                                continue
-                            # this String is actually a _file_
-                            # try to determine the file content-type
-                            fn = value.filename.split('\\')[-1]
-                            if propdef.has_key('name'):
-                                props['name'] = fn
-                            # use this info as the type/filename properties
-                            if propdef.has_key('type'):
-                                if hasattr(value, 'type') and value.type:
-                                    props['type'] = value.type
-                                elif mimetypes.guess_type(fn)[0]:
-                                    props['type'] = mimetypes.guess_type(fn)[0]
-                                else:
-                                    props['type'] = "application/octet-stream"
-                            # finally, read the content RAW
-                            value = value.value
-                        else:
-                            value = hyperdb.rawToHyperdb(self.db, cl,
-                                nodeid, propname, value)
-
+                    # Try handling file upload
+                    if (isinstance(proptype, hyperdb.String) and
+                        hasattr(value, 'filename') and
+                        value.filename is not None):
+                        value = self.parse_file(propdef, props, value)
                     else:
                         value = hyperdb.rawToHyperdb(self.db, cl, nodeid,
-                            propname, value)
-                except hyperdb.HyperdbValueError, msg:
-                    raise FormError, msg
+                                                     propname, value)
+                except hyperdb.HyperdbValueError as msg:
+                    raise FormError(msg)
 
             # register that we got this property
             if isinstance(proptype, hyperdb.Multilink):
@@ -483,9 +502,9 @@ class FormParser:
                 except KeyError:
                     # this might be a new property for which there is
                     # no existing value
-                    if not propdef.has_key(propname):
+                    if propname not in propdef:
                         raise
-                except IndexError, message:
+                except IndexError as message:
                     raise FormError(str(message))
 
                 # make sure the existing multilink is sorted.  We must
@@ -494,15 +513,17 @@ class FormParser:
                 # The canonical order (given in Multilink.from_raw) is
                 # by the numeric value of the IDs.
                 if isinstance(proptype, hyperdb.Multilink):
-                    existing.sort(lambda x, y: cmp(int(x),int(y)))
+                    existing.sort(key=int)
 
                 # "missing" existing values may not be None
                 if not existing:
                     if isinstance(proptype, hyperdb.String):
-                        # some backends store "missing" Strings as empty strings
+                        # some backends store "missing" Strings as
+                        # empty strings
                         if existing == self.db.BACKEND_MISSING_STRING:
                             existing = None
-                    elif isinstance(proptype, hyperdb.Number):
+                    elif isinstance(proptype, hyperdb.Number) or \
+                         isinstance(proptype, hyperdb.Integer):
                         # some backends store "missing" Numbers as 0 :(
                         if existing == self.db.BACKEND_MISSING_NUMBER:
                             existing = None
@@ -525,9 +546,10 @@ class FormParser:
 
                 props[propname] = value
 
-        # check to see if we need to specially link a file to the note
+        # check to see if we need to specially link files to the note
         if have_note and have_file:
-            all_links.append(('msg', '-1', 'files', [('file', '-1')]))
+            for fid in have_file:
+                all_links.append(('msg', '-1', 'files', [('file', fid)]))
 
         # see if all the required properties have been supplied
         s = []
@@ -535,7 +557,7 @@ class FormParser:
             # register the values we got
             got = got_props.get(thing, {})
             for entry in required[:]:
-                if got.has_key(entry):
+                if entry in got:
                     required.remove(entry)
 
             # If a user doesn't have edit permission for a given property,
@@ -565,29 +587,64 @@ class FormParser:
                 'property': ', '.join(map(self.gettext, required))
             })
         if s:
-            raise FormError, '\n'.join(s)
+            raise FormError('\n'.join(s))
 
         # When creating a FileClass node, it should have a non-empty content
         # property to be created. When editing a FileClass node, it should
         # either have a non-empty content property or no property at all. In
         # the latter case, nothing will change.
-        for (cn, id), props in all_props.items():
+        for (cn, id), props in list(all_props.items()):
             if id is not None and id.startswith('-') and not props:
                 # new item (any class) with no content - ignore
                 del all_props[(cn, id)]
             elif isinstance(self.db.classes[cn], hyperdb.FileClass):
-                if id is not None and id.startswith('-'):
-                    if not props.get('content', ''):
+                # three cases:
+                # id references existng file. If content is empty,
+                #    remove content from form so we don't wipe
+                #    existing file contents.
+                # id is -1, -2 ... I.E. a new file.
+                #  if content is not defined remove all fields that
+                #     reference that file.
+                #  if content is defined, let it pass through even if
+                #     content is empty. Yes people can upload/create
+                #     empty files.
+                if 'content' in props:
+                    if id is not None and \
+                       not id.startswith('-') and \
+                       not props['content']:
+                        # This is an existing file with emtpy content
+                        # value in the form.
+                        del props['content']
+                else:
+                    # this is a new file without any content property.
+                    if id is not None and id.startswith('-'):
                         del all_props[(cn, id)]
-                elif props.has_key('content') and not props['content']:
-                    raise FormError, self._('File is empty')
+                # if this is a new file with content (even 0 length content)
+                # allow it through and create the zero length file.
         return all_props, all_links
 
+    def parse_file(self, fpropdef, fprops, v):
+        # try to determine the file content-type
+        fn = v.filename.split('\\')[-1]
+        if 'name' in fpropdef:
+            fprops['name'] = fn
+        # use this info as the type/filename properties
+        if 'type' in fpropdef:
+            if hasattr(v, 'type') and v.type:
+                fprops['type'] = v.type
+            elif mimetypes.guess_type(fn)[0]:
+                fprops['type'] = mimetypes.guess_type(fn)[0]
+            else:
+                fprops['type'] = "application/octet-stream"
+        # finally, read the content RAW
+        return v.value
+
     def extractFormList(self, value):
         ''' Extract a list of values from the form value.
 
             It may be one of:
-             [MiniFieldStorage('value'), MiniFieldStorage('value','value',...), ...]
+             [MiniFieldStorage('value'),
+             MiniFieldStorage('value','value',...), ...]
              MiniFieldStorage('value,value,...')
              MiniFieldStorage('value')
         '''
@@ -603,6 +660,6 @@ class FormParser:
         value = [i.strip() for i in values.split(',')]
 
         # filter out the empty bits
-        return filter(None, value)
+        return list(filter(None, value))
 
 # vim: set et sts=4 sw=4 :
