@@ -35,11 +35,11 @@ the dynamic portion of a message really means.
 """
 __docformat__ = 'restructuredtext'
 
-import errno
 import gettext as gettext_module
 import os
 
 from roundup import msgfmt
+from roundup.anypy.strings import is_us
 
 # List of directories for mo file search (see SF bug 1219689)
 LOCALE_DIRS = [
@@ -58,36 +58,39 @@ if _mo_path not in LOCALE_DIRS:
     LOCALE_DIRS.append(_mo_path)
 del _mo_path
 
+import sys
+# __file__ should be something like:
+#    /usr/local/lib/python3.10/site-packages/roundup/i18n.py
+# os.prefix should be /usr, /usr/local or root of virtualenv
+#    strip leading / to make os.path.join work right.
+path = __file__
+for N in 1, 2:
+    path = os.path.dirname(path)
+    # path is /usr/local/lib/python3.10/site-packages
+_ldir = os.path.join(path, sys.prefix[1:], 'share', 'locale')
+if os.path.isdir(_ldir):
+    LOCALE_DIRS.append(_ldir)
+# try other places locale files are hidden on install
+_ldir = os.path.join(path, sys.prefix[1:], 'local', 'share', 'locale')
+if os.path.isdir(_ldir):
+    LOCALE_DIRS.append(_ldir)
+try:
+    _ldir = os.path.join(path, sys.base_prefix[1:], 'local', 'share', 'locale')
+    if os.path.isdir(_ldir):
+        LOCALE_DIRS.append(_ldir)
+    _ldir = os.path.join(path, sys.base_prefix[1:], 'share', 'locale')
+    if os.path.isdir(_ldir):
+        LOCALE_DIRS.append(_ldir)
+except AttributeError:
+    pass # no base_prefix on 2.7
+del _ldir
+
 # Roundup text domain
 DOMAIN = "roundup"
 
-if hasattr(gettext_module.GNUTranslations, "ngettext"):
-    # gettext_module has everything needed
-    RoundupNullTranslations = gettext_module.NullTranslations
-    RoundupTranslations = gettext_module.GNUTranslations
-else:
-    # prior to 2.3, there was no plural forms.  mix simple emulation in
-    class PluralFormsMixIn:
-        def ngettext(self, singular, plural, count):
-            if count == 1:
-                _msg = singular
-            else:
-                _msg = plural
-            return self.gettext(_msg)
-        def ungettext(self, singular, plural, count):
-            if count == 1:
-                _msg = singular
-            else:
-                _msg = plural
-            return self.ugettext(_msg)
-    class RoundupNullTranslations(
-        gettext_module.NullTranslations, PluralFormsMixIn
-    ):
-        pass
-    class RoundupTranslations(
-        gettext_module.GNUTranslations, PluralFormsMixIn
-    ):
-        pass
+RoundupNullTranslations = gettext_module.NullTranslations
+RoundupTranslations = gettext_module.GNUTranslations
+
 
 def find_locales(language=None):
     """Return normalized list of locale names to try for given language
@@ -104,7 +107,7 @@ def find_locales(language=None):
             if val:
                 languages = val.split(':')
                 break
-    elif isinstance(language, str) or  isinstance(language, unicode):
+    elif is_us(language):
         languages = [language]
     else:
         # 'language' must be iterable
@@ -117,6 +120,7 @@ def find_locales(language=None):
                 nelangs.append(nelang)
     return nelangs
 
+
 def get_mofile(languages, localedir, domain=None):
     """Return the first of .mo files found in localedir for languages
 
@@ -161,7 +165,8 @@ def get_mofile(languages, localedir, domain=None):
         # see what we've found
         if motime < potime:
             # compile
-            msgfmt.make(pofile, mofile)
+            mo = msgfmt.Msgfmt(pofile).get()
+            open(mofile, 'wb').write(mo)
         elif motime == 0:
             # no files found - proceed to the next locale name
             continue
@@ -169,10 +174,10 @@ def get_mofile(languages, localedir, domain=None):
         return mofile
     return None
 
+
 def get_translation(language=None, tracker_home=None,
-    translation_class=RoundupTranslations,
-    null_translation_class=RoundupNullTranslations
-):
+                    translation_class=RoundupTranslations,
+                    null_translation_class=RoundupNullTranslations):
     """Return Translation object for given language and domain
 
     Argument 'language' may be a single language code or a list of codes.
@@ -206,22 +211,45 @@ def get_translation(language=None, tracker_home=None,
             mofiles.append(get_mofile(locales, system_locale, DOMAIN))
     # filter out elements that are not found
     mofiles = filter(None, mofiles)
-    if mofiles:
-        translator = translation_class(open(mofiles[0], "rb"))
-        for mofile in mofiles[1:]:
-            # note: current implementation of gettext_module
-            #   always adds fallback to the end of the fallback chain.
-            translator.add_fallback(translation_class(open(mofile, "rb")))
-    else:
+    translator = None
+    for mofile in mofiles:
+        try:
+            mo = open(mofile, "rb")
+            if translator is None:
+                translator = translation_class(mo)
+                # the .mo file this translator loaded from
+                translator._file = mofile
+            else:
+                # note: current implementation of gettext_module
+                #   always adds fallback to the end of the fallback chain.
+                fallback = translation_class(mo)
+                fallback._file = mofile
+                translator.add_fallback(fallback)
+            mo.close()
+        except IOError:
+            # ignore unreadable .mo files
+            pass
+    if translator is None:
         translator = null_translation_class()
     return translator
 
+
 # static translations object
 translation = get_translation()
 # static translation functions
 _ = gettext = translation.gettext
-ugettext = translation.ugettext
+try:
+    # Python 2.
+    ugettext = translation.ugettext
+except AttributeError:
+    # Python 3.
+    ugettext = translation.gettext
 ngettext = translation.ngettext
-ungettext = translation.ungettext
+try:
+    # Python 2.
+    ungettext = translation.ungettext
+except AttributeError:
+    # Python 3.
+    ungettext = translation.ngettext
 
 # vim: set filetype=python sts=4 sw=4 et si :
