@@ -1,4 +1,3 @@
-from __future__ import nested_scopes
 #
 # Copyright (c) 2001 Bizar Software Pty Ltd (http://www.bizarsoftware.com.au/)
 # This module is free software, and you may redistribute it and/or modify
@@ -21,39 +20,44 @@ from __future__ import nested_scopes
 """
 __docformat__ = 'restructuredtext'
 
-import re, os, smtplib, socket, time, random
-import cStringIO, base64, mimetypes
-import os.path
+import time
+import base64, mimetypes
 import logging
-from email import Encoders
-from email.Utils import formataddr
-from email.Header import Header
-from email.MIMEText import MIMEText
-from email.MIMEBase import MIMEBase
-from email.MIMEMultipart import MIMEMultipart
-
-from anypy.email_ import FeedParser
+from email import encoders
+from email.parser import FeedParser
+from email.utils import formataddr
+from email.header import Header
+from email.mime.text import MIMEText
+from email.mime.base import MIMEBase
+from email.mime.multipart import MIMEMultipart
 
 from roundup import password, date, hyperdb
 from roundup.i18n import _
 from roundup.hyperdb import iter_roles
 
-from roundup.mailer import Mailer, MessageSendError, encode_quopri, \
-    nice_sender_header
+from roundup.mailer import Mailer, MessageSendError, nice_sender_header
+from roundup.i18n import RoundupNullTranslations
+
+from roundup.anypy.strings import b2s, s2u
+import roundup.anypy.random_ as random_
 
 try:
-    import pyme, pyme.core
+    import gpg, gpg.core
 except ImportError:
-    pyme = None
+    gpg = None
 
 
-class Database:
+class Database(object):
 
     # remember the journal uid for the current journaltag so that:
     # a. we don't have to look it up every time we need it, and
     # b. if the journaltag disappears during a transaction, we don't barf
     #    (eg. the current user edits their username)
     journal_uid = None
+
+    def __init__(self):
+        self.i18n = RoundupNullTranslations()
+
     def getuid(self):
         """Return the id of the "user" node associated with the user
         that owns this connection to the hyperdatabase."""
@@ -119,14 +123,30 @@ class Database:
         cl = self.user
 
         props['roles'] = self.config.NEW_WEB_USER_ROLES
-        userid = cl.create(**props)
-        # clear the props from the otk database
+        try:
+            # ASSUME:: ValueError raised during create due to key value
+            # conflict. I an use message in exception to determine
+            # when I should intercept the exception with a more
+            # friendly error message. If i18n is used to translate
+            # original exception message this will fail and translated
+            # text (probably unfriendly) will be used.
+            userid = cl.create(**props)
+        except ValueError as e:
+            username = props['username']
+            # Try to make error message less cryptic to the user.
+            if str(e) == 'node with key "%s" exists' % username:
+                raise ValueError(
+                    _("Username '%s' already exists." % username))
+            else:
+                raise
+
+            # clear the props from the otk database
         self.getOTKManager().destroy(otk)
+        # commit cl.create (and otk changes)
         self.commit()
 
         return userid
 
-
     def log_debug(self, msg, *args, **kwargs):
         """Log a message with level DEBUG."""
 
@@ -155,19 +175,19 @@ class Database:
             We allow to register user-defined cache-clearing routines
             that are called by this routine.
         """
-        if getattr (self, 'cache_callbacks', None) :
+        if getattr(self, 'cache_callbacks', None):
             for method, param in self.cache_callbacks:
                 method(param)
 
-    def registerClearCacheCallback(self, method, param = None):
+    def registerClearCacheCallback(self, method, param=None):
         """ Register a callback method for clearing the cache.
             It is called with the given param as the only parameter.
             Even if the parameter is not specified, the method has to
             accept a single parameter.
         """
-        if not getattr (self, 'cache_callbacks', None) :
+        if not getattr(self, 'cache_callbacks', None):
             self.cache_callbacks = []
-        self.cache_callbacks.append ((method, param))
+        self.cache_callbacks.append((method, param))
 
 
 class DetectorError(RuntimeError):
@@ -175,6 +195,7 @@ class DetectorError(RuntimeError):
     """
     pass
 
+
 # deviation from spec - was called IssueClass
 class IssueClass:
     """This class is intended to be mixed-in with a hyperdb backend
@@ -221,7 +242,9 @@ class IssueClass:
         """
 
     def nosymessage(self, issueid, msgid, oldvalues, whichnosy='nosy',
-            from_address=None, cc=[], bcc=[], cc_emails = [], bcc_emails = []):
+                    from_address=None, cc=[], bcc=[], cc_emails=[],
+                    bcc_emails=[], subject=None,
+                    note_filter=None, add_headers={}):
         """Send a message to the members of an issue's nosy list.
 
         The message is sent only to users on the nosy list who are not
@@ -232,6 +255,12 @@ class IssueClass:
         If 'msgid' is None, the message gets sent only to the nosy
         list, and it's called a 'System Message'.
 
+        The "subject" argument is used as subject for the message. If no
+        subject is passed, a subject will be generated from the message.
+        Note the subject does not include the item designator [classID]
+        prefix that allows proper processing of reply emails. The caller
+        needs to include that label in the subject if needed.
+
         The "cc" argument indicates additional recipients to send the
         message to that may not be specified in the message's recipients
         list.
@@ -242,12 +271,12 @@ class IssueClass:
         address lists. Note that the list of bcc users *is* updated in
         the recipient list of the message, so this field has to be
         protected (using appropriate permissions), otherwise the bcc
-        will be decuceable for users who have web access to the tracker.
+        will be deduceable for users who have web access to the tracker.
 
         The cc_emails and bcc_emails arguments take a list of additional
         recipient email addresses (just the mail address not roundup users)
         this can be useful for sending to additional email addresses
-        which are no roundup users. These arguments are currently not
+        which are not roundup users. These arguments are currently not
         used by roundups nosyreaction but can be used by customized
         (nosy-)reactors.
 
@@ -256,6 +285,14 @@ class IssueClass:
         defined, we try to send encrypted mail to *all* users
         *including* cc, bcc, cc_emails and bcc_emails and this might
         fail if not all the keys are available in roundups keyring.
+
+        If note_filter is specified it is a function with this
+        prototype:
+            note_filter(original_note, issueid, newvalues, oldvalues)
+        If called, note_filter returns the new value for the message body.
+
+        The add_headers parameter allows to set additional headers for
+        the outgoing email.
         """
         encrypt = self.db.config.PGP_ENABLE and self.db.config.PGP_ENCRYPT
         pgproles = self.db.config.PGP_ROLES
@@ -267,8 +304,8 @@ class IssueClass:
             authid = None
             recipients = []
 
-        sendto = dict (plain = [], crypt = [])
-        bcc_sendto = dict (plain = [], crypt = [])
+        sendto = dict(plain=[], crypt=[])
+        bcc_sendto = dict(plain=[], crypt=[])
         seen_message = {}
         for recipient in recipients:
             seen_message[recipient] = 1
@@ -299,7 +336,7 @@ class IssueClass:
                             'View', userid, 'msg', prop, msgid)
             return (userid and
                     (self.db.user.get(userid, 'username') != 'anonymous') and
-                    allowed and not seen_message.has_key(userid))
+                    allowed and userid not in seen_message)
 
         # possibly send the message to the author, as long as they aren't
         # anonymous
@@ -307,7 +344,7 @@ class IssueClass:
             (self.db.config.MESSAGES_TO_AUTHOR == 'yes' or
              (self.db.config.MESSAGES_TO_AUTHOR == 'new' and not oldvalues) or
              (self.db.config.MESSAGES_TO_AUTHOR == 'nosy' and authid in
-             self.get(issueid, whichnosy)))):
+              self.get(issueid, whichnosy)))):
             add_recipient(authid, sendto)
 
         if authid:
@@ -317,24 +354,30 @@ class IssueClass:
         for userid in cc + self.get(issueid, whichnosy):
             if good_recipient(userid):
                 add_recipient(userid, sendto)
+                seen_message[userid] = 1
         if encrypt and not pgproles:
-            sendto['crypt'].extend (cc_emails)
+            sendto['crypt'].extend(cc_emails)
         else:
-            sendto['plain'].extend (cc_emails)
+            sendto['plain'].extend(cc_emails)
 
         # now deal with bcc people.
         for userid in bcc:
             if good_recipient(userid):
                 add_recipient(userid, bcc_sendto)
+                seen_message[userid] = 1
         if encrypt and not pgproles:
-            bcc_sendto['crypt'].extend (bcc_emails)
+            bcc_sendto['crypt'].extend(bcc_emails)
         else:
-            bcc_sendto['plain'].extend (bcc_emails)
+            bcc_sendto['plain'].extend(bcc_emails)
 
         if oldvalues:
             note = self.generateChangeNote(issueid, oldvalues)
         else:
             note = self.generateCreateNote(issueid)
+        if note_filter:
+            cn = self.classname
+            cl = self.db.classes[cn]
+            note = note_filter(note, issueid, self.db, cl, oldvalues)
 
         # If we have new recipients, update the message's recipients
         # and send the mail.
@@ -344,10 +387,12 @@ class IssueClass:
                 self.db.msg.set(msgid, recipients=recipients)
         if sendto['plain'] or bcc_sendto['plain']:
             self.send_message(issueid, msgid, note, sendto['plain'],
-                from_address, bcc_sendto['plain'])
+                              from_address, bcc_sendto['plain'],
+                              subject, add_headers=add_headers)
         if sendto['crypt'] or bcc_sendto['crypt']:
             self.send_message(issueid, msgid, note, sendto['crypt'],
-                from_address, bcc_sendto['crypt'], crypt=True)
+                              from_address, bcc_sendto['crypt'], subject,
+                              crypt=True, add_headers=add_headers)
 
     # backwards compatibility - don't remove
     sendmessage = nosymessage
@@ -356,9 +401,9 @@ class IssueClass:
         """ Encrypt given message to sendto receivers.
             Returns a new RFC 3156 conforming message.
         """
-        plain = pyme.core.Data(message.as_string())
-        cipher = pyme.core.Data()
-        ctx = pyme.core.Context()
+        plain = gpg.core.Data(message.as_string())
+        cipher = gpg.core.Data()
+        ctx = gpg.core.Context()
         ctx.set_armor(1)
         keys = []
         for adr in sendto:
@@ -368,13 +413,13 @@ class IssueClass:
             if k is not None:
                 keys.append(k)
             else:
-                msg = _('No key for "%(adr)s" in keyring')%locals()
-                raise MessageSendError, msg
+                msg = _('No key for "%(adr)s" in keyring') % locals()
+                raise MessageSendError(msg)
             ctx.op_keylist_end()
         ctx.op_encrypt(keys, 1, plain, cipher)
-        cipher.seek(0,0)
+        cipher.seek(0, 0)
         msg = MIMEMultipart('encrypted', boundary=None, _subparts=None,
-            protocol="application/pgp-encrypted")
+                            protocol="application/pgp-encrypted")
         part = MIMEBase('application', 'pgp-encrypted')
         part.set_payload("Version: 1\r\n")
         msg.attach(part)
@@ -384,9 +429,11 @@ class IssueClass:
         return msg
 
     def send_message(self, issueid, msgid, note, sendto, from_address=None,
-            bcc_sendto=[], crypt=False):
+                     bcc_sendto=[], subject=None, crypt=False,
+                     add_headers={}, authid=None):
         '''Actually send the nominated message from this issue to the sendto
-           recipients, with the note appended.
+           recipients, with the note appended. It's possible to add
+           headers to the message with the add_headers variable.
         '''
         users = self.db.user
         messages = self.db.msg
@@ -403,18 +450,20 @@ class IssueClass:
         if not messageid:
             # this is an old message that didn't get a messageid, so
             # create one
-            messageid = "<%s.%s.%s%s@%s>"%(time.time(), random.random(),
-                                           self.classname, issueid,
-                                           self.db.config.MAIL_DOMAIN)
+            messageid = "<%s.%s.%s%s@%s>" % (time.time(),
+                b2s(base64.b32encode(random_.token_bytes(10))),
+                self.classname, issueid, self.db.config['MAIL_DOMAIN'])
             if msgid is not None:
                 messages.set(msgid, messageid=messageid)
 
         # compose title
         cn = self.classname
-        title = self.get(issueid, 'title') or '%s message copy'%cn
+        title = self.get(issueid, 'title') or '%s message copy' % cn
 
         # figure author information
-        if msgid:
+        if authid:
+            pass
+        elif msgid:
             authid = messages.get(msgid, 'author')
         else:
             authid = self.db.getuid()
@@ -424,7 +473,7 @@ class IssueClass:
         authaddr = users.get(authid, 'address', '')
 
         if authaddr and self.db.config.MAIL_ADD_AUTHOREMAIL:
-            authaddr = " <%s>" % formataddr( ('',authaddr) )
+            authaddr = " <%s>" % formataddr(('', authaddr))
         elif authaddr:
             authaddr = ""
 
@@ -439,10 +488,10 @@ class IssueClass:
         if authid and self.db.config.MAIL_ADD_AUTHORINFO:
             if msgid and len(self.get(issueid, 'messages')) == 1:
                 m.append(_("New submission from %(authname)s%(authaddr)s:")
-                    % locals())
+                         % locals())
             elif msgid:
                 m.append(_("%(authname)s%(authaddr)s added the comment:")
-                    % locals())
+                         % locals())
             else:
                 m.append(_("Change by %(authname)s%(authaddr)s:") % locals())
             m.append('')
@@ -453,8 +502,8 @@ class IssueClass:
 
         # get the files for this message
         message_files = []
-        if msgid :
-            for fileid in messages.get(msgid, 'files') :
+        if msgid:
+            for fileid in messages.get(msgid, 'files'):
                 # check the attachment size
                 filesize = self.db.filesize('file', fileid, None)
                 if filesize <= self.db.config.NOSY_MAX_ATTACHMENT_SIZE:
@@ -464,7 +513,8 @@ class IssueClass:
                     link = "".join((base, files.classname, fileid))
                     filename = files.get(fileid, 'name')
                     m.append(_("File '%(filename)s' not attached - "
-                        "you can download it from %(link)s.") % locals())
+                               "you can download it from %(link)s.") %
+                             locals())
 
         # add the change note
         if note:
@@ -478,7 +528,7 @@ class IssueClass:
         charset = getattr(self.db.config, 'EMAIL_CHARSET', 'utf-8')
 
         # construct the content and convert to unicode object
-        body = unicode('\n'.join(m), 'utf-8').encode(charset)
+        body = s2u('\n'.join(m))
 
         # make sure the To line is always the same (for testing mostly)
         sendto.sort()
@@ -492,7 +542,9 @@ class IssueClass:
         if from_tag:
             from_tag = ' ' + from_tag
 
-        subject = '[%s%s] %s'%(cn, issueid, title)
+        if subject is None:
+            subject = '[%s%s] %s' % (cn, issueid, title)
+
         author = (authname + from_tag, from_address)
 
         # send an individual message per recipient?
@@ -502,9 +554,9 @@ class IssueClass:
             sendto = [sendto]
 
         # tracker sender info
-        tracker_name = unicode(self.db.config.TRACKER_NAME, 'utf-8')
+        tracker_name = s2u(self.db.config.TRACKER_NAME)
         tracker_name = nice_sender_header(tracker_name, from_address,
-            charset)
+                                          charset)
 
         # now send one or more messages
         # TODO: I believe we have to create a new message each time as we
@@ -517,8 +569,21 @@ class IssueClass:
 
             message = mailer.get_standard_message(multipart=message_files)
 
-            # set reply-to to the tracker
-            message['Reply-To'] = tracker_name
+            # set reply-to as requested by config option
+            # TRACKER_REPLYTO_ADDRESS
+            replyto_config = self.db.config.TRACKER_REPLYTO_ADDRESS
+            if replyto_config:
+                if replyto_config == "AUTHOR":
+                    # note that authaddr at this point is already
+                    # surrounded by < >, so get the original address
+                    # from the db as nice_send_header adds < >
+                    replyto_addr = nice_sender_header(authname,
+                        users.get(authid, 'address', ''), charset)
+                else:
+                    replyto_addr = replyto_config
+            else:
+                replyto_addr = tracker_name
+            message['Reply-To'] = replyto_addr
 
             # message ids
             if messageid:
@@ -532,7 +597,18 @@ class IssueClass:
                 if not isinstance(prop, (hyperdb.Link, hyperdb.Multilink)):
                     continue
                 cl = self.db.getclass(prop.classname)
-                if not 'name' in cl.getprops():
+                label = None
+                if 'name' in cl.getprops():
+                    label = 'name'
+                if prop.msg_header_property in cl.getprops():
+                    label = prop.msg_header_property
+                if prop.msg_header_property == "":
+                    # if msg_header_property is set to empty string
+                    # suppress the header entirely. You can't use
+                    # 'msg_header_property == None'. None is the
+                    # default value.
+                    label = None
+                if not label:
                     continue
                 if isinstance(prop, hyperdb.Link):
                     value = self.get(issueid, propname)
@@ -543,14 +619,33 @@ class IssueClass:
                     values = self.get(issueid, propname)
                     if not values:
                         continue
-                values = [cl.get(v, 'name') for v in values]
+                values = [cl.get(v, label) for v in values]
                 values = ', '.join(values)
-                header = "X-Roundup-%s-%s"%(self.classname, propname)
+                header = "X-Roundup-%s-%s" % (self.classname, propname)
                 try:
-                    message[header] = values.encode('ascii')
+                    values.encode('ascii')
+                    message[header] = values
                 except UnicodeError:
                     message[header] = Header(values, charset)
 
+            # Add header for main id number to make filtering
+            # email easier than extracting from subject line.
+            header = "X-Roundup-%s-Id" % (self.classname)
+            values = issueid
+            try:
+                values.encode('ascii')
+                message[header] = values
+            except UnicodeError:
+                message[header] = Header(values, charset)
+            # Generate additional headers
+            for k in add_headers:
+                v = add_headers[k]
+                try:
+                    v.encode('ascii')
+                    message[k] = v
+                except UnicodeError:
+                    message[k] = Header(v, charset)
+
             if not inreplyto:
                 # Default the reply to the first message
                 msgs = self.get(issueid, 'messages')
@@ -565,28 +660,31 @@ class IssueClass:
             # attach files
             if message_files:
                 # first up the text as a part
-                part = MIMEText(body)
-                part.set_charset(charset)
-                encode_quopri(part)
+                part = mailer.get_standard_message()
+                part.set_payload(body, part.get_charset())
                 message.attach(part)
 
                 for fileid in message_files:
                     name = files.get(fileid, 'name')
-                    mime_type = files.get(fileid, 'type')
-                    content = files.get(fileid, 'content')
+                    mime_type = (files.get(fileid, 'type') or
+                                 mimetypes.guess_type(name)[0] or
+                                 'application/octet-stream')
                     if mime_type == 'text/plain':
+                        content = files.get(fileid, 'content')
+                        part = MIMEText('')
+                        del part['Content-Transfer-Encoding']
                         try:
-                            content.decode('ascii')
+                            enc = content.encode('ascii')
+                            part = mailer.get_text_message('us-ascii')
+                            part.set_payload(enc)
                         except UnicodeError:
                             # the content cannot be 7bit-encoded.
                             # use quoted printable
                             # XXX stuffed if we know the charset though :(
-                            part = MIMEText(content)
-                            encode_quopri(part)
-                        else:
-                            part = MIMEText(content)
-                            part['Content-Transfer-Encoding'] = '7bit'
+                            part = mailer.get_text_message('utf-8')
+                            part.set_payload(content, part.get_charset())
                     elif mime_type == 'message/rfc822':
+                        content = files.get(fileid, 'content')
                         main, sub = mime_type.split('/')
                         p = FeedParser()
                         p.feed(content)
@@ -594,43 +692,41 @@ class IssueClass:
                         part.set_payload([p.close()])
                     else:
                         # some other type, so encode it
-                        if not mime_type:
-                            # this should have been done when the file was saved
-                            mime_type = mimetypes.guess_type(name)[0]
-                        if mime_type is None:
-                            mime_type = 'application/octet-stream'
+                        content = files.get(fileid, 'binary_content')
                         main, sub = mime_type.split('/')
                         part = MIMEBase(main, sub)
                         part.set_payload(content)
-                        Encoders.encode_base64(part)
+                        encoders.encode_base64(part)
                     cd = 'Content-Disposition'
-                    part[cd] = 'attachment;\n filename="%s"'%name
+                    part[cd] = 'attachment;\n filename="%s"' % name
                     message.attach(part)
 
             else:
-                message.set_payload(body)
-                encode_quopri(message)
+                message.set_payload(body, message.get_charset())
 
             if crypt:
-                send_msg = self.encrypt_to (message, sendto)
+                send_msg = self.encrypt_to(message, sendto)
             else:
                 send_msg = message
             mailer.set_message_attributes(send_msg, sendto, subject, author)
-            send_msg ['Message-Id'] = message ['Message-Id']
-            send_msg ['Reply-To'] = message ['Reply-To']
-            if message.get ('In-Reply-To'):
-                send_msg ['In-Reply-To'] = message ['In-Reply-To']
-            mailer.smtp_send(sendto, send_msg.as_string())
+            if crypt:
+                send_msg['Message-Id'] = message['Message-Id']
+                send_msg['Reply-To'] = message['Reply-To']
+                if message.get('In-Reply-To'):
+                    send_msg['In-Reply-To'] = message['In-Reply-To']
+
+            if sendto:
+                mailer.smtp_send(sendto, send_msg.as_string())
             if first:
                 if crypt:
                     # send individual bcc mails, otherwise receivers can
                     # deduce bcc recipients from keys in message
                     for bcc in bcc_sendto:
-                        send_msg = self.encrypt_to (message, [bcc])
-                        send_msg ['Message-Id'] = message ['Message-Id']
-                        send_msg ['Reply-To'] = message ['Reply-To']
-                        if message.get ('In-Reply-To'):
-                            send_msg ['In-Reply-To'] = message ['In-Reply-To']
+                        send_msg = self.encrypt_to(message, [bcc])
+                        send_msg['Message-Id'] = message['Message-Id']
+                        send_msg['Reply-To'] = message['Reply-To']
+                        if message.get('In-Reply-To'):
+                            send_msg['In-Reply-To'] = message['In-Reply-To']
                         mailer.smtp_send([bcc], send_msg.as_string())
                 elif bcc_sendto:
                     mailer.smtp_send(bcc_sendto, send_msg.as_string())
@@ -642,7 +738,7 @@ class IssueClass:
         # simplistic check to see if the url is valid,
         # then append a trailing slash if it is missing
         base = self.db.config.TRACKER_WEB
-        if (not isinstance(base , type('')) or
+        if (not isinstance(base, type('')) or
             not (base.startswith('http://') or base.startswith('https://'))):
             web = "Configuration Error: TRACKER_WEB isn't a " \
                 "fully-qualified URL"
@@ -653,11 +749,10 @@ class IssueClass:
 
         # ensure the email address is properly quoted
         email = formataddr((self.db.config.TRACKER_NAME,
-            self.db.config.TRACKER_EMAIL))
+                            self.db.config.TRACKER_EMAIL))
 
         line = '_' * max(len(web)+2, len(email))
-        return '\n%s\n%s\n<%s>\n%s'%(line, email, web, line)
-
+        return '\n%s\n%s\n<%s>\n%s' % (line, email, web, line)
 
     def generateCreateNote(self, issueid):
         """Generate a create note that lists initial property values
@@ -668,9 +763,11 @@ class IssueClass:
 
         # list the values
         m = []
-        prop_items = props.items()
-        prop_items.sort()
+        prop_items = sorted(props.items())
         for propname, prop in prop_items:
+            # Omit quiet properties from history/changelog
+            if prop.quiet:
+                continue
             value = cl.get(issueid, propname, None)
             # skip boring entries
             if not value:
@@ -696,7 +793,7 @@ class IssueClass:
                 value = str(value)
                 if '\n' in value:
                     value = '\n'+self.indentChangeNoteValue(value)
-            m.append('%s: %s'%(propname, value))
+            m.append('%s: %s' % (propname, value))
         m.insert(0, '----------')
         m.insert(0, '')
         return '\n'.join(m)
@@ -705,8 +802,8 @@ class IssueClass:
         """Generate a change note that lists property changes
         """
         if not isinstance(oldvalues, type({})):
-            raise TypeError("'oldvalues' must be dict-like, not %s."%
-                type(oldvalues))
+            raise TypeError("'oldvalues' must be dict-like, not %s." %
+                            type(oldvalues))
 
         cn = self.classname
         cl = self.db.classes[cn]
@@ -715,7 +812,7 @@ class IssueClass:
 
         # determine what changed
         for key in oldvalues.keys():
-            if key in ['files','messages']:
+            if key in ['files', 'messages']:
                 continue
             if key in ('actor', 'activity', 'creator', 'creation'):
                 continue
@@ -729,20 +826,22 @@ class IssueClass:
             # this happens when property was added
             try:
                 old_value = oldvalues[key]
-                if type(new_value) is type([]):
+                if isinstance(new_value, type([])):
                     new_value.sort()
                     old_value.sort()
                 if new_value != old_value:
                     changed[key] = old_value
-            except:
+            except Exception:
                 changed[key] = new_value
 
         # list the changes
         m = []
-        changed_items = changed.items()
-        changed_items.sort()
+        changed_items = sorted(changed.items())
         for propname, oldvalue in changed_items:
             prop = props[propname]
+            # Omit quiet properties from history/changelog
+            if prop.quiet:
+                continue
             value = cl.get(issueid, propname, None)
             if isinstance(prop, hyperdb.Link):
                 link = self.db.classes[prop.classname]
@@ -756,7 +855,7 @@ class IssueClass:
                         oldvalue = link.get(oldvalue, key)
                     else:
                         oldvalue = ''
-                change = '%s -> %s'%(oldvalue, value)
+                change = '%s -> %s' % (oldvalue, value)
             elif isinstance(prop, hyperdb.Multilink):
                 change = ''
                 if value is None: value = []
@@ -773,7 +872,7 @@ class IssueClass:
                         l.append(entry)
                 if l:
                     l.sort()
-                    change = '+%s'%(', '.join(l))
+                    change = '+%s' % (', '.join(l))
                     l = []
                 # check for removals
                 for entry in oldvalue:
@@ -784,15 +883,15 @@ class IssueClass:
                         l.append(entry)
                 if l:
                     l.sort()
-                    change += ' -%s'%(', '.join(l))
+                    change += ' -%s' % (', '.join(l))
             else:
-                change = '%s -> %s'%(oldvalue, value)
+                change = '%s -> %s' % (oldvalue, value)
                 if '\n' in change:
                     value = self.indentChangeNoteValue(str(value))
                     oldvalue = self.indentChangeNoteValue(str(oldvalue))
                     change = _('\nNow:\n%(new)s\nWas:\n%(old)s') % {
                         "new": value, "old": oldvalue}
-            m.append('%s: %s'%(propname, change))
+            m.append('%s: %s' % (propname, change))
         if m:
             m.insert(0, '----------')
             m.insert(0, '')
@@ -800,7 +899,7 @@ class IssueClass:
 
     def indentChangeNoteValue(self, text):
         lines = text.rstrip('\n').split('\n')
-        lines = [ '  '+line for line in lines ]
+        lines = ['  '+line for line in lines]
         return '\n'.join(lines)
 
 # vim: set filetype=python sts=4 sw=4 et si :
