@@ -17,6 +17,7 @@
 
 """Command-line script that runs a server over roundup.cgi.client.
 """
+from __future__ import print_function
 __docformat__ = 'restructuredtext'
 
 
@@ -35,19 +36,36 @@ if (osp.exists(thisdir + '/__init__.py') and
 # --/
 
 
-import errno, cgi, getopt, os, socket, sys, traceback, urllib, time
-import ConfigParser, BaseHTTPServer, SocketServer, StringIO
+import errno, getopt, io, os, socket, sys, traceback, time
+
+try:
+    # Python 3.
+    import socketserver
+except ImportError:
+    # Python 2.
+    import SocketServer as socketserver
+
+try:
+    # Python 2.
+    reload
+except NameError:
+    # Python 3.
+    from importlib import reload
 
 try:
     from OpenSSL import SSL
 except ImportError:
     SSL = None
 
+from roundup.anypy.html import html_escape
+
 # python version check
 from roundup import configuration, version_check
 from roundup import __version__ as roundup_version
 
 # Roundup modules of use here
+from roundup.anypy import http_, urllib_
+from roundup.anypy.strings import s2b, StringIO
 from roundup.cgi import cgitb, client
 from roundup.cgi.PageTemplates.PageTemplate import PageTemplate
 import roundup.instance
@@ -56,7 +74,7 @@ from roundup.i18n import _
 # "default" favicon.ico
 # generate by using "icotool" and tools/base64
 import zlib, base64
-favico = zlib.decompress(base64.decodestring('''
+favico = zlib.decompress(base64.b64decode(b'''
 eJztjr1PmlEUh59XgVoshdYPWorFIhaRFq0t9pNq37b60lYSTRzcTFw6GAfj5gDYaF0dTB0MxMSE
 gQQd3FzKJiEC0UCIUUN1M41pV2JCXySg/0ITn5tfzvmdc+85FwT56HSc81UJjXJsk1UsNcsSqCk1
 BS64lK+vr7OyssLJyQl2ux2j0cjU1BQajYZIJEIwGMRms+H3+zEYDExOTjI2Nsbm5iZWqxWv18vW
@@ -77,7 +95,7 @@ DEFAULT_PORT = 8080
 # "debug" means "none" + no tracker/template cache
 MULTIPROCESS_TYPES = ["debug", "none"]
 try:
-    import thread
+    import threading  # nosrc: F401
 except ImportError:
     pass
 else:
@@ -86,34 +104,36 @@ if hasattr(os, 'fork'):
     MULTIPROCESS_TYPES.append("fork")
 DEFAULT_MULTIPROCESS = MULTIPROCESS_TYPES[-1]
 
+
 def auto_ssl():
-    print _('WARNING: generating temporary SSL certificate')
+    print(_('WARNING: generating temporary SSL certificate'))
     import OpenSSL, random
     pkey = OpenSSL.crypto.PKey()
-    pkey.generate_key(OpenSSL.crypto.TYPE_RSA, 768)
+    pkey.generate_key(OpenSSL.crypto.TYPE_RSA, 2048)
     cert = OpenSSL.crypto.X509()
-    cert.set_serial_number(random.randint(0, sys.maxint))
+    cert.set_serial_number(random.randint(0, sys.maxsize))
     cert.gmtime_adj_notBefore(0)
-    cert.gmtime_adj_notAfter(60 * 60 * 24 * 365) # one year
+    cert.gmtime_adj_notAfter(60 * 60 * 24 * 365)  # one year
     cert.get_subject().CN = '*'
     cert.get_subject().O = 'Roundup Dummy Certificate'
     cert.get_issuer().CN = 'Roundup Dummy Certificate Authority'
     cert.get_issuer().O = 'Self-Signed'
     cert.set_pubkey(pkey)
-    cert.sign(pkey, 'md5')
-    ctx = SSL.Context(SSL.SSLv23_METHOD)
+    cert.sign(pkey, 'sha512')
+    ctx = SSL.Context(OpenSSL.SSL.TLSv1_2_METHOD)
     ctx.use_privatekey(pkey)
     ctx.use_certificate(cert)
 
     return ctx
 
-class SecureHTTPServer(BaseHTTPServer.HTTPServer):
+
+class SecureHTTPServer(http_.server.HTTPServer):
     def __init__(self, server_address, HandlerClass, ssl_pem=None):
         assert SSL, "pyopenssl not installed"
-        BaseHTTPServer.HTTPServer.__init__(self, server_address, HandlerClass)
+        http_.server.HTTPServer.__init__(self, server_address, HandlerClass)
         self.socket = socket.socket(self.address_family, self.socket_type)
         if ssl_pem:
-            ctx = SSL.Context(SSL.SSLv23_METHOD)
+            ctx = SSL.Context(SSL.TLSv1_2_METHOD)
             ctx.use_privatekey_file(ssl_pem)
             ctx.use_certificate_file(ssl_pem)
         else:
@@ -158,6 +178,7 @@ class SecureHTTPServer(BaseHTTPServer.HTTPServer):
                     which the HTTP handlers require """
                 def __init__(self, conn):
                     self.__conn = conn
+
                 def makefile(self, mode, bufsize):
                     fo = socket._fileobject(self.__conn, mode, bufsize)
                     return RetryingFile(fo)
@@ -168,7 +189,8 @@ class SecureHTTPServer(BaseHTTPServer.HTTPServer):
             conn = ConnFixer(conn)
         return (conn, info)
 
-class RoundupRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
+
+class RoundupRequestHandler(http_.server.BaseHTTPRequestHandler):
     TRACKER_HOMES = {}
     TRACKERS = None
     LOG_IPADDRESS = 1
@@ -207,7 +229,7 @@ class RoundupRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
         except client.NotFound:
             self.send_error(404, self.path)
         except client.Unauthorised as message:
-            self.send_error(403, '%s (%s)'%(self.path, message))
+            self.send_error(403, '%s (%s)' % (self.path, message))
         except:
             exc, val, tb = sys.exc_info()
             if hasattr(socket, 'timeout') and isinstance(val, socket.timeout):
@@ -221,33 +243,34 @@ class RoundupRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
                 if self.DEBUG_MODE:
                     try:
                         reload(cgitb)
-                        self.wfile.write(cgitb.breaker())
-                        self.wfile.write(cgitb.html())
+                        self.wfile.write(s2b(cgitb.breaker()))
+                        self.wfile.write(s2b(cgitb.html()))
                     except:
-                        s = StringIO.StringIO()
+                        s = StringIO()
                         traceback.print_exc(None, s)
-                        self.wfile.write("<pre>")
-                        self.wfile.write(cgi.escape(s.getvalue()))
-                        self.wfile.write("</pre>\n")
+                        self.wfile.write(b"<pre>")
+                        self.wfile.write(s2b(html_escape(s.getvalue())))
+                        self.wfile.write(b"</pre>\n")
                 else:
                     # user feedback
-                    self.wfile.write(cgitb.breaker())
+                    self.wfile.write(s2b(cgitb.breaker()))
                     ts = time.ctime()
-                    self.wfile.write('''<p>%s: An error occurred. Please check
-                    the server log for more information.</p>'''%ts)
+                    self.wfile.write(s2b('''<p>%s: An error occurred. Please check
+                    the server log for more information.</p>''' % ts))
                     # out to the logfile
-                    print 'EXCEPTION AT', ts
+                    print('EXCEPTION AT', ts)
                     traceback.print_exc()
 
-    do_GET = do_POST = do_HEAD = run_cgi
+    do_GET = do_POST = do_HEAD = do_PUT = do_DELETE = \
+                       do_PATCH = do_OPTIONS = run_cgi
 
     def index(self):
         ''' Print up an index of the available trackers
         '''
-        keys = self.TRACKER_HOMES.keys()
+        keys = list(self.TRACKER_HOMES.keys())
         if len(keys) == 1:
             self.send_response(302)
-            self.send_header('Location', urllib.quote(keys[0]) + '/index')
+            self.send_header('Location', urllib_.quote(keys[0]) + '/index')
             self.end_headers()
         else:
             self.send_response(200)
@@ -260,21 +283,21 @@ class RoundupRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
             template = open(self.CONFIG['TEMPLATE']).read()
             pt = PageTemplate()
             pt.write(template)
-            extra = { 'trackers': self.TRACKERS,
-                'nothing' : None,
-                'true' : 1,
-                'false' : 0,
-            }
-            w(pt.pt_render(extra_context=extra))
+            extra = {'trackers': self.TRACKERS,
+                     'nothing': None,
+                     'true': 1,
+                     'false': 0,
+                    }
+            w(s2b(pt.pt_render(extra_context=extra)))
         else:
-            w(_('<html><head><title>Roundup trackers index</title></head>\n'
-                '<body><h1>Roundup trackers index</h1><ol>\n'))
+            w(s2b(_('<html><head><title>Roundup trackers index</title></head>\n'
+                    '<body><h1>Roundup trackers index</h1><ol>\n')))
             keys.sort()
             for tracker in keys:
-                w('<li><a href="%(tracker_url)s/index">%(tracker_name)s</a>\n'%{
-                    'tracker_url': urllib.quote(tracker),
-                    'tracker_name': cgi.escape(tracker)})
-            w('</ol></body></html>')
+                w(s2b('<li><a href="%(tracker_url)s/index">%(tracker_name)s</a>\n' % {
+                    'tracker_url': urllib_.quote(tracker),
+                    'tracker_name': html_escape(tracker)}))
+            w(b'</ol></body></html>')
 
     def inner_run_cgi(self):
         ''' This is the inner part of the CGI handling
@@ -293,16 +316,18 @@ class RoundupRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
                 if os.access(favicon_filepath, os.R_OK):
                     favicon_fileobj = open(favicon_filepath, 'rb')
 
-
             if favicon_fileobj is None:
-                favicon_fileobj = StringIO.StringIO(favico)
+                favicon_fileobj = io.BytesIO(favico)
 
             self.send_response(200)
             self.send_header('Content-Type', 'image/x-icon')
+            self.send_header('Content-Length', len(favico))
+            self.send_header('Cache-Control', "public, max-age=86400")
             self.end_headers()
 
-            # this bufsize is completely arbitrary, I picked 4K because it sounded good.
-            # if someone knows of a better buffer size, feel free to plug it in.
+            # this bufsize is completely arbitrary, I picked 4K because
+            # it sounded good. if someone knows of a better buffer size,
+            # feel free to plug it in.
             bufsize = 4 * 1024
             Processing = True
             while Processing:
@@ -331,19 +356,19 @@ class RoundupRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
 
         # figure the tracker
         l_path = rest.split('/')
-        tracker_name = urllib.unquote(l_path[1]).lower()
+        tracker_name = urllib_.unquote(l_path[1]).lower()
 
         # handle missing trailing '/'
         if len(l_path) == 2:
             self.send_response(301)
             # redirect - XXX https??
             protocol = 'http'
-            url = '%s://%s%s/'%(protocol, self.headers['host'], rest)
+            url = '%s://%s%s/' % (protocol, self.headers['host'], rest)
             if query:
-               url += '?' + query
+                url += '?' + query
             self.send_header('Location', url)
             self.end_headers()
-            self.wfile.write('Moved Permanently')
+            self.wfile.write(b'Moved Permanently')
             return
 
         # figure out what the rest of the path is
@@ -356,36 +381,55 @@ class RoundupRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
         env = {}
         env['TRACKER_NAME'] = tracker_name
         env['REQUEST_METHOD'] = self.command
-        env['PATH_INFO'] = urllib.unquote(rest)
+        env['PATH_INFO'] = urllib_.unquote(rest)
         if query:
             env['QUERY_STRING'] = query
-        if self.headers.typeheader is None:
-            env['CONTENT_TYPE'] = self.headers.type
+        if hasattr(self.headers, 'get_content_type'):
+            # Python 3.  We need the raw header contents.
+            content_type = self.headers.get('content-type')
+        elif self.headers.typeheader is None:
+            # Python 2.
+            content_type = self.headers.type
         else:
-            env['CONTENT_TYPE'] = self.headers.typeheader
-        length = self.headers.getheader('content-length')
+            # Python 2.
+            content_type = self.headers.typeheader
+        if content_type:
+            env['CONTENT_TYPE'] = content_type
+        length = self.headers.get('content-length')
         if length:
             env['CONTENT_LENGTH'] = length
-        co = filter(None, self.headers.getheaders('cookie'))
+        if hasattr(self.headers, 'get_all'):
+            # Python 3.
+            ch = self.headers.get_all('cookie', [])
+        else:
+            # Python 2.
+            ch = self.headers.getheaders('cookie')
+        co = list(filter(None, ch))
         if co:
             env['HTTP_COOKIE'] = ', '.join(co)
-        env['HTTP_AUTHORIZATION'] = self.headers.getheader('authorization')
+        env['HTTP_AUTHORIZATION'] = self.headers.get('authorization')
+        # self.CONFIG['INCLUDE_HEADERS'] is a list.
+        for h in self.CONFIG['INCLUDE_HEADERS']:
+            env[h] = self.headers.get(h, None)
+            # if header is MISSING
+            if env[h] is None:
+                del(env[h])
         env['SCRIPT_NAME'] = ''
         env['SERVER_NAME'] = self.server.server_name
         env['SERVER_PORT'] = str(self.server.server_port)
         try:
-            env['HTTP_HOST'] = self.headers ['host']
+            env['HTTP_HOST'] = self.headers['host']
         except KeyError:
             env['HTTP_HOST'] = ''
         # https://tools.ietf.org/html/draft-ietf-appsawg-http-forwarded-10
         # headers.
-        xfh = self.headers.getheader('X-Forwarded-Host', None)
+        xfh = self.headers.get('X-Forwarded-Host', None)
         if xfh:
             # If behind a proxy, this is the hostname supplied
             # via the Host header to the proxy. Used by core code.
             # Controlled by the CSRF settings.
-            env['HTTP_X-FORWARDED-HOST'] = xfh
-        xff = self.headers.getheader('X-Forwarded-For', None)
+            env['HTTP_X_FORWARDED_HOST'] = xfh
+        xff = self.headers.get('X-Forwarded-For', None)
         if xff:
             # xff is a list of ip addresses for original client/proxies:
             # X-Forwarded-For: clientIP, proxy1IP, proxy2IP
@@ -394,8 +438,8 @@ class RoundupRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
             # Made available for extensions if the user trusts it.
             # E.g. you may wish to disable recaptcha validation extension
             # if the ip of the client matches 172.16.0.0.
-            env['HTTP_X-FORWARDED-FOR'] = xff
-        xfp = self.headers.getheader('X-Forwarded-Proto', None)
+            env['HTTP_X_FORWARDED_FOR'] = xff
+        xfp = self.headers.get('X-Forwarded-Proto', None)
         if xfp:
             # xfp is the protocol (http/https) seen by proxies in the
             # path of the request. I am not sure if there is only
@@ -408,8 +452,8 @@ class RoundupRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
             # May not be trustworthy. Do not use in core without
             # config option to control its use.
             # Made available for extensions if the user trusts it.
-            env['HTTP_X-FORWARDED-PROTO'] = xfp
-        if os.environ.has_key('CGI_SHOW_TIMING'):
+            env['HTTP_X_FORWARDED_PROTO'] = xfp
+        if 'CGI_SHOW_TIMING' in os.environ:
             env['CGI_SHOW_TIMING'] = os.environ['CGI_SHOW_TIMING']
         env['HTTP_ACCEPT_LANGUAGE'] = self.headers.get('accept-language')
         referer = self.headers.get('Referer')
@@ -420,10 +464,13 @@ class RoundupRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
             env['HTTP_ORIGIN'] = origin
         xrw = self.headers.get('x-requested-with')
         if xrw:
-            env['HTTP_X-REQUESTED-WITH'] = xrw
-        range = self.headers.getheader('range')
+            env['HTTP_X_REQUESTED_WITH'] = xrw
+        range = self.headers.get('range')
         if range:
             env['HTTP_RANGE'] = range
+        if_range = self.headers.get('if-range')
+        if range:
+            env['HTTP_IF_RANGE'] = if_range
 
         # do the roundup thing
         tracker = self.get_tracker(tracker_name)
@@ -445,10 +492,10 @@ class RoundupRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
             logger.info("%s - - [%s] %s" %
                         (self.client_address[0],
                          self.log_date_time_string(),
-                         format%args))
+                         format % args))
         else:
             try:
-                BaseHTTPServer.BaseHTTPRequestHandler.log_message(self,
+                http_.server.BaseHTTPRequestHandler.log_message(self,
                                                          format, *args)
             except IOError:
                 # stderr is no longer viable
@@ -460,9 +507,12 @@ class RoundupRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
             self.send_header(key, value)
         self.end_headers()
 
+
 def error():
     exc_type, exc_value = sys.exc_info()[:2]
-    return _('Error: %s: %s' % (exc_type, exc_value))
+    return _('Error: %(type)s: %(value)s' % {'type': exc_type,
+                                             'value': exc_value})
+
 
 def setgid(group):
     if group is None:
@@ -472,13 +522,13 @@ def setgid(group):
 
     # if root, setgid to the running user
     if os.getuid():
-        print _('WARNING: ignoring "-g" argument, not root')
+        print(_('WARNING: ignoring "-g" argument, not root'))
         return
 
     try:
         import grp
     except ImportError:
-        raise ValueError, _("Can't change groups - no grp module")
+        raise ValueError(_("Can't change groups - no grp module"))
     try:
         try:
             gid = int(group)
@@ -487,9 +537,10 @@ def setgid(group):
         else:
             grp.getgrgid(gid)
     except KeyError:
-        raise ValueError,_("Group %(group)s doesn't exist")%locals()
+        raise ValueError(_("Group %(group)s doesn't exist") % locals())
     os.setgid(gid)
 
+
 def setuid(user):
     if not hasattr(os, 'getuid'):
         return
@@ -498,16 +549,16 @@ def setuid(user):
     if user is None:
         if os.getuid():
             return
-        raise ValueError, _("Can't run as root!")
+        raise ValueError(_("Can't run as root!"))
 
     if os.getuid():
-        print _('WARNING: ignoring "-u" argument, not root')
+        print(_('WARNING: ignoring "-u" argument, not root'))
         return
 
     try:
         import pwd
     except ImportError:
-        raise ValueError, _("Can't change users - no pwd module")
+        raise ValueError(_("Can't change users - no pwd module"))
     try:
         try:
             uid = int(user)
@@ -516,9 +567,10 @@ def setuid(user):
         else:
             pwd.getpwuid(uid)
     except KeyError:
-        raise ValueError, _("User %(user)s doesn't exist")%locals()
+        raise ValueError(_("User %(user)s doesn't exist") % locals())
     os.setuid(uid)
 
+
 class TrackerHomeOption(configuration.FilePathOption):
 
     # Tracker homes do not need any description strings
@@ -528,6 +580,7 @@ class TrackerHomeOption(configuration.FilePathOption):
                 "value": self.value2str(self._value),
             }
 
+
 class ServerConfig(configuration.Config):
 
     SETTINGS = (
@@ -584,6 +637,15 @@ class ServerConfig(configuration.Config):
             (configuration.NullableFilePathOption, "pem", "",
                 "PEM file used for SSL. A temporary self-signed certificate\n"
                 "will be used if left blank."),
+            (configuration.WordListOption, "include_headers", "",
+                "Comma separated list of extra headers that should\n"
+                "be copied into the CGI environment.\n"
+                "E.G. if you want to access the REMOTE_USER and\n"
+                "X-Proxy-User headers in the back end,\n"
+                "set to the value REMOTE_USER,X-Proxy-User."),
+            (configuration.HttpVersionOption, "http_version", "HTTP/1.1",
+                "Change to HTTP/1.0 if needed. This disables keepalive."),
+
         )),
         ("trackers", (), "Roundup trackers to serve.\n"
             "Each option in this section defines single Roundup tracker.\n"
@@ -608,6 +670,8 @@ class ServerConfig(configuration.Config):
         "loghttpvialogger": 'L',
         "ssl": "s",
         "pem": "e:",
+        "include_headers": "I:",
+        "http_version": 'V:',
     }
 
     def __init__(self, config_file=None):
@@ -622,14 +686,13 @@ class ServerConfig(configuration.Config):
             return
         # config defaults appear in all sections.
         # filter them out.
-        defaults = config.defaults().keys()
+        defaults = list(config.defaults().keys())
         for name in config.options("trackers"):
             if name not in defaults:
                 self.add_option(TrackerHomeOption(self, "trackers", name))
 
     def getopt(self, args, short_options="", long_options=(),
-        config_load_options=("C", "config"), **options
-    ):
+               config_load_options=("C", "config"), **options):
         options.update(self.OPTIONS)
         return configuration.Config.getopt(self, args,
             short_options, long_options, config_load_options, **options)
@@ -647,8 +710,9 @@ class ServerConfig(configuration.Config):
 
     def set_logging(self):
         """Initialise logging to the configured file, if any."""
-        # appending, unbuffered
-        sys.stdout = sys.stderr = open(self["LOGFILE"], 'a', 0)
+        # appending, line-buffered (Python 3 does not allow unbuffered
+        # text files)
+        sys.stdout = sys.stderr = open(self["LOGFILE"], 'a', 1)
 
     def get_server(self):
         """Return HTTP server object to run"""
@@ -661,7 +725,7 @@ class ServerConfig(configuration.Config):
             trackers = None
         else:
             trackers = dict([(name, roundup.instance.open(home, optimize=1))
-                for (name, home) in tracker_homes])
+                             for (name, home) in tracker_homes])
 
         # build customized request handler class
         class RequestHandler(RoundupRequestHandler):
@@ -677,6 +741,8 @@ class ServerConfig(configuration.Config):
                     # internal state correctly so that later closing SSL
                     # socket works (with SSL end-handshake started)
                     self.request.do_handshake()
+                RoundupRequestHandler.protocol_version = \
+                                         self.CONFIG["HTTP_VERSION"]
                 RoundupRequestHandler.setup(self)
 
             def finish(self):
@@ -693,23 +759,23 @@ class ServerConfig(configuration.Config):
             # socket, so we do this only for non-SSL connections.
             if hasattr(socket, 'setdefaulttimeout'):
                 socket.setdefaulttimeout(60)
-            base_server = BaseHTTPServer.HTTPServer
+            base_server = http_.server.HTTPServer
 
         # obtain request server class
         if self["MULTIPROCESS"] not in MULTIPROCESS_TYPES:
-            print _("Multiprocess mode \"%s\" is not available, "
-                "switching to single-process") % self["MULTIPROCESS"]
+            print(_("Multiprocess mode \"%s\" is not available, "
+                    "switching to single-process") % self["MULTIPROCESS"])
             self["MULTIPROCESS"] = "none"
             server_class = base_server
         elif self["MULTIPROCESS"] == "fork":
-            class ForkingServer(SocketServer.ForkingMixIn,
-                base_server):
-                    pass
+            class ForkingServer(socketserver.ForkingMixIn,
+                                base_server):
+                pass
             server_class = ForkingServer
         elif self["MULTIPROCESS"] == "thread":
-            class ThreadingServer(SocketServer.ThreadingMixIn,
-                base_server):
-                    pass
+            class ThreadingServer(socketserver.ThreadingMixIn,
+                                  base_server):
+                pass
             server_class = ThreadingServer
         else:
             server_class = base_server
@@ -723,10 +789,18 @@ class ServerConfig(configuration.Config):
                 kwargs['ssl_pem'] = self["PEM"]
             httpd = server_class(*args, **kwargs)
         except socket.error as e:
-            if e[0] == errno.EADDRINUSE:
-                raise socket.error, \
-                    _("Unable to bind to port %s, port already in use.") \
-                    % self["PORT"]
+            if e.args[0] == errno.EADDRINUSE:
+                raise socket.error(_("Unable to bind to port %s, "
+                                     "port already in use.") % self["PORT"])
+            if e.args[0] == errno.EACCES:
+                raise socket.error(_("Unable to bind to port %(port)s, "
+                                     "access not allowed, "
+                                     "errno: %(errno)s %(msg)s" % {
+                                         "port": self["PORT"],
+                                         "errno": e.args[0],
+                                         "msg": e.args[1] }
+                ))
+                                     
             raise
         # change user and/or group
         setgid(self["GROUP"])
@@ -734,9 +808,10 @@ class ServerConfig(configuration.Config):
         # return the server
         return httpd
 
+
 try:
     import win32serviceutil
-except:
+except ImportError:
     RoundupService = None
 else:
 
@@ -761,7 +836,7 @@ else:
             (optlist, args) = config.getopt(sys.argv[1:])
             if not config["LOGFILE"]:
                 servicemanager.LogMsg(servicemanager.EVENTLOG_ERROR_TYPE,
-                    servicemanager.PYS_SERVICE_STOPPED,
+                                      servicemanager.PYS_SERVICE_STOPPED,
                     (self._svc_display_name_, "\r\nMissing logfile option"))
                 self.ReportServiceStatus(win32service.SERVICE_STOPPED)
                 return
@@ -770,13 +845,15 @@ else:
             self.running = 1
             self.ReportServiceStatus(win32service.SERVICE_RUNNING)
             servicemanager.LogMsg(servicemanager.EVENTLOG_INFORMATION_TYPE,
-                servicemanager.PYS_SERVICE_STARTED, (self._svc_display_name_,
-                    " at %s:%s" % (config["HOST"], config["PORT"])))
+                                  servicemanager.PYS_SERVICE_STARTED,
+                                  (self._svc_display_name_,
+                                   " at %s:%s" % (config["HOST"],
+                                                  config["PORT"])))
             while self.running:
                 self.server.handle_request()
             servicemanager.LogMsg(servicemanager.EVENTLOG_INFORMATION_TYPE,
-                servicemanager.PYS_SERVICE_STOPPED,
-                (self._svc_display_name_, ""))
+                                  servicemanager.PYS_SERVICE_STOPPED,
+                                  (self._svc_display_name_, ""))
             self.ReportServiceStatus(win32service.SERVICE_STOPPED)
 
         def SvcStop(self):
@@ -790,6 +867,7 @@ else:
             sock.close()
             self.ReportServiceStatus(win32service.SERVICE_STOP_PENDING)
 
+
 def usage(message=''):
     if RoundupService:
         os_part = \
@@ -806,8 +884,8 @@ def usage(message=''):
                to the file indicated by PIDfile. The -l option *must* be
                specified if -d is used.'''
     if message:
-        message += '\n'
-    print _('''%(message)sUsage: roundup-server [options] [name=tracker home]*
+        message += '\n\n'
+    print(_('''\n%(message)sUsage: roundup-server [options] [name=tracker home]*
 
 Options:
  -v            print the Roundup version number and exit
@@ -819,6 +897,7 @@ Options:
                connections, defaults to localhost, use 0.0.0.0 to bind
                to all network interfaces
  -p <port>     set the port to listen on (default: %(port)s)
+ -I <header1[,header2]*> list of headers to pass to the backend
  -l <fname>    log to the file indicated by fname instead of stderr/stdout
  -N            log client machine names instead of IP addresses (much slower)
  -i <fname>    set tracker index template
@@ -827,6 +906,9 @@ Options:
  -e <fname>    PEM file containing SSL key and certificate
  -t <mode>     multiprocess mode (default: %(mp_def)s).
                Allowed values: %(mp_types)s.
+ -V <version>  set HTTP version (default: HTTP/1.1).
+               Allowed values: HTTP/1.0, HTTP/1.1.
+
 %(os_part)s
 
 Long options:
@@ -869,7 +951,7 @@ How to use "name=tracker home":
     "port": DEFAULT_PORT,
     "mp_def": DEFAULT_MULTIPROCESS,
     "mp_types": ", ".join(MULTIPROCESS_TYPES),
-}
+})
 
 
 def writepidfile(pidfile):
@@ -880,6 +962,7 @@ def writepidfile(pidfile):
         pidfile.write(str(pid))
         pidfile.close()
 
+
 def daemonize(pidfile):
     ''' Turn this process into a daemon.
         - make sure the sys.std(in|out|err) are completely cut off
@@ -914,24 +997,37 @@ def daemonize(pidfile):
     os.dup2(devnull, 1)
     os.dup2(devnull, 2)
 
+
 undefined = []
+
+
 def run(port=undefined, success_message=None):
     ''' Script entry point - handle args and figure out what to to.
     '''
     config = ServerConfig()
     # additional options
-    short_options = "hvS"
-    if RoundupService:
-        short_options += 'c'
+    short_options = "hvSc"
     try:
         (optlist, args) = config.getopt(sys.argv[1:],
-            short_options, ("help", "version", "save-config",))
+                                        short_options,
+                                        ("help", "version", "save-config",))
     except (getopt.GetoptError, configuration.ConfigurationError) as e:
         usage(str(e))
         return
 
     # if running in windows service mode, don't do any other stuff
     if ("-c", "") in optlist:
+        global RoundupService
+        if not RoundupService:
+            RoundupService = True  # make sure usage displays -c help text
+            error_m = """
+ERROR: -c is not available because roundup couldn't import
+   win32serviceutil from pywin32. See Installation docs
+   for pywin32 details.
+            """
+            usage(error_m)
+            return
+
         # acquire command line options recognized by service
         short_options = "cC:"
         long_options = ["config"]
@@ -955,7 +1051,7 @@ def run(port=undefined, success_message=None):
         RoundupService._exe_args_ = " ".join(svc_args)
         # pass the control to serviceutil
         win32serviceutil.HandleCommandLine(RoundupService,
-            argv=sys.argv[:1] + args)
+                                           argv=sys.argv[:1] + args)
         return
 
     # add tracker names from command line.
@@ -965,21 +1061,21 @@ def run(port=undefined, success_message=None):
             try:
                 name, home = arg.split('=')
             except ValueError:
-                raise ValueError, _("Instances must be name=home")
+                raise ValueError(_("Instances must be name=home"))
             config.add_option(TrackerHomeOption(config, "trackers", name))
             config["TRACKERS_" + name.upper()] = home
 
     # handle remaining options
     if optlist:
-        for (opt, arg) in optlist:
+        for (opt, _arg) in optlist:
             if opt in ("-h", "--help"):
                 usage()
             elif opt in ("-v", "--version"):
-                print '%s (python %s)' % (roundup_version,
-                    sys.version.split()[0])
+                print('%s (python %s)' % (roundup_version,
+                                          sys.version.split()[0]))
             elif opt in ("-S", "--save-config"):
                 config.save()
-                print _("Configuration saved to %s") % config.filepath
+                print(_("Configuration saved to %s") % config.filepath)
         # any of the above options prevent server from running
         return
 
@@ -997,8 +1093,8 @@ def run(port=undefined, success_message=None):
     # fork the server from our parent if a pidfile is specified
     if config["PIDFILE"]:
         if not hasattr(os, 'fork'):
-            print _("Sorry, you can't run the server as a daemon"
-                " on this Operating System")
+            print(_("Sorry, you can't run the server as a daemon"
+                    " on this Operating System"))
             sys.exit(0)
         else:
             if config['NODAEMON']:
@@ -1007,18 +1103,25 @@ def run(port=undefined, success_message=None):
                 daemonize(config["PIDFILE"])
 
     # create the server
-    httpd = config.get_server()
+    try:
+        httpd = config.get_server()
+    except Exception as e:
+        # capture all exceptions and pretty print them
+        print(e)
+        sys.exit(2)
 
     if success_message:
-        print success_message
+        print(success_message)
     else:
-        print _('Roundup server started on %(HOST)s:%(PORT)s') \
-            % config
+        print(_('Roundup server started on %(HOST)s:%(PORT)s')
+              % config)
 
     try:
         httpd.serve_forever()
     except KeyboardInterrupt:
-        print 'Keyboard Interrupt: exiting'
+        print('Keyboard Interrupt: exiting')
+        httpd.socket.close()
+
 
 if __name__ == '__main__':
     run()
