@@ -18,8 +18,9 @@
 import unittest, os, shutil, time
 from roundup.backends import get_backend
 
-from db_test_base import DBTest, ROTest, SchemaTest, ClassicInitTest, config
-from db_test_base import HTMLItemTest, SpecialActionTest
+from .db_test_base import DBTest, ROTest, SchemaTest, ClassicInitTest, config
+from .db_test_base import HTMLItemTest, SpecialActionTest
+from .rest_common import TestCase as RestTestCase
 
 class anydbmOpener:
     module = get_backend('anydbm')
@@ -48,7 +49,7 @@ class anydbmHTMLItemTest(HTMLItemTest, unittest.TestCase):
     backend = 'anydbm'
 
 
-from session_common import SessionTest
+from .session_common import SessionTest
 class anydbmSessionTest(anydbmOpener, SessionTest, unittest.TestCase):
     pass
 
@@ -56,4 +57,7 @@ class anydbmSpecialActionTestCase(anydbmOpener, SpecialActionTest,
                                   unittest.TestCase):
     backend = 'anydbm'
 
+class anydbmRestTest (RestTestCase, unittest.TestCase):
+    backend = 'anydbm'
+
 # vim: set filetype=python ts=4 sw=4 et si
