@@ -16,11 +16,14 @@
 # SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 
 import unittest, os, shutil, time
+import sqlite3 as sqlite
+
 from roundup.backends import get_backend, have_backend
 
-from db_test_base import DBTest, ROTest, SchemaTest, ClassicInitTest, config
-from db_test_base import ConcurrentDBTest, FilterCacheTest
-from db_test_base import SpecialActionTest
+from .db_test_base import DBTest, ROTest, SchemaTest, ClassicInitTest, config
+from .db_test_base import ConcurrentDBTest, FilterCacheTest
+from .db_test_base import SpecialActionTest
+from .rest_common  import TestCase as RestTestCase
 
 class sqliteOpener:
     if have_backend('sqlite'):
@@ -31,8 +34,43 @@ class sqliteOpener:
 
 
 class sqliteDBTest(sqliteOpener, DBTest, unittest.TestCase):
-    pass
 
+    def testUpgrade_6_to_7(self):
+
+        # load the database
+        self.db.issue.create(title="flebble frooz")
+        self.db.commit()
+
+        if self.db.database_schema['version'] != 7:
+            self.skipTest("This test only runs for database version 7")
+
+        self.db.database_schema['version'] = 6
+
+        # dropping _fts
+        #  select * from _fts
+        #    it should fail.
+        # run post-init
+        #    same select should succeed (with no rows returned)
+
+        self.db.sql("drop table __fts")
+
+        with self.assertRaises(sqlite.OperationalError) as ctx:
+            self.db.sql("select * from __fts")
+
+        self.assertIn("no such table: __fts", ctx.exception.args[0])
+
+        if hasattr(self, "downgrade_only"):
+            return
+
+        # test upgrade adding __fts table
+        self.db.post_init()
+
+        # select should now work.
+        self.db.sql("select * from __fts")
+
+        # we should be at the current db version
+        self.assertEqual(self.db.database_schema['version'],
+                         self.db.current_db_version)
 
 class sqliteROTest(sqliteOpener, ROTest, unittest.TestCase):
     pass
@@ -58,6 +96,9 @@ class sqliteSpecialActionTestCase(sqliteOpener, SpecialActionTest,
     backend = 'sqlite'
 
 
-from session_common import SessionTest
+from .session_common import SessionTest
 class sqliteSessionTest(sqliteOpener, SessionTest, unittest.TestCase):
     pass
+
+class sqliteRestTest (RestTestCase, unittest.TestCase):
+    backend = 'sqlite'
